@@ -89,14 +89,14 @@ enum
   PROP_CHANNEL_NEW_NAME,
   PROP_CHANNEL_NEW_COLOR,
 
-  PROP_PATH_NEW_NAME,
+  PROP_VECTORS_NEW_NAME,
 
-  PROP_PATH_EXPORT_PATH,
-  PROP_PATH_EXPORT_ACTIVE_ONLY,
+  PROP_VECTORS_EXPORT_PATH,
+  PROP_VECTORS_EXPORT_ACTIVE_ONLY,
 
-  PROP_PATH_IMPORT_PATH,
-  PROP_PATH_IMPORT_MERGE,
-  PROP_PATH_IMPORT_SCALE,
+  PROP_VECTORS_IMPORT_PATH,
+  PROP_VECTORS_IMPORT_MERGE,
+  PROP_VECTORS_IMPORT_SCALE,
 
   PROP_SELECTION_FEATHER_RADIUS,
   PROP_SELECTION_FEATHER_EDGE_LOCK,
@@ -422,50 +422,50 @@ gimp_dialog_config_class_init (GimpDialogConfigClass *klass)
                           "channel-new-color",
                           "Default new channel color and opacity",
                           CHANNEL_NEW_COLOR_BLURB,
-                          TRUE, half_transparent,
+                          half_transparent,
                           GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_STRING (object_class, PROP_PATH_NEW_NAME,
+  GIMP_CONFIG_PROP_STRING (object_class, PROP_VECTORS_NEW_NAME,
                            "path-new-name",
                            "Default new path name",
-                           PATH_NEW_NAME_BLURB,
+                           VECTORS_NEW_NAME_BLURB,
                            _("Path"),
                            GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_PATH (object_class, PROP_PATH_EXPORT_PATH,
+  GIMP_CONFIG_PROP_PATH (object_class, PROP_VECTORS_EXPORT_PATH,
                          "path-export-path",
                          "Default path export folder path",
-                         PATH_EXPORT_PATH_BLURB,
+                         VECTORS_EXPORT_PATH_BLURB,
                          GIMP_CONFIG_PATH_FILE,
                          NULL,
                          GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_PATH_EXPORT_ACTIVE_ONLY,
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_VECTORS_EXPORT_ACTIVE_ONLY,
                             "path-export-active-only",
                             "Default export only the selected paths",
-                            PATH_EXPORT_ACTIVE_ONLY_BLURB,
+                            VECTORS_EXPORT_ACTIVE_ONLY_BLURB,
                             TRUE,
                             GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_PATH (object_class, PROP_PATH_IMPORT_PATH,
+  GIMP_CONFIG_PROP_PATH (object_class, PROP_VECTORS_IMPORT_PATH,
                          "path-import-path",
                          "Default path import folder path",
-                         PATH_IMPORT_PATH_BLURB,
+                         VECTORS_IMPORT_PATH_BLURB,
                          GIMP_CONFIG_PATH_FILE,
                          NULL,
                          GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_PATH_IMPORT_MERGE,
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_VECTORS_IMPORT_MERGE,
                             "path-import-merge",
-                            "Default merge imported path",
-                            PATH_IMPORT_MERGE_BLURB,
+                            "Default merge imported vectors",
+                            VECTORS_IMPORT_MERGE_BLURB,
                             FALSE,
                             GIMP_PARAM_STATIC_STRINGS);
 
-  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_PATH_IMPORT_SCALE,
+  GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_VECTORS_IMPORT_SCALE,
                             "path-import-scale",
-                            "Default scale imported path",
-                            PATH_IMPORT_SCALE_BLURB,
+                            "Default scale imported vectors",
+                            VECTORS_IMPORT_SCALE_BLURB,
                             FALSE,
                             GIMP_PARAM_STATIC_STRINGS);
 
@@ -473,13 +473,7 @@ gimp_dialog_config_class_init (GimpDialogConfigClass *klass)
                            "selection-feather-radius",
                            "Selection feather radius",
                            SELECTION_FEATHER_RADIUS_BLURB,
-                           /* NOTE: the max value is the max value of
-                            * "std-dev-x|y" arguments in operation
-                            * "gegl:gaussian-blur", multiplied by magic
-                            * number 3.5 (see gimp_gegl_apply_feather())
-                            * code).
-                            */
-                            0.0, 5250.0, 5.0,
+                           0.0, 32767.0, 5.0,
                            GIMP_PARAM_STATIC_STRINGS);
 
   GIMP_CONFIG_PROP_BOOLEAN (object_class, PROP_SELECTION_FEATHER_EDGE_LOCK,
@@ -598,9 +592,9 @@ gimp_dialog_config_finalize (GObject *object)
   g_clear_pointer (&config->color_profile_path,  g_free);
   g_clear_pointer (&config->layer_new_name,      g_free);
   g_clear_pointer (&config->channel_new_name,    g_free);
-  g_clear_pointer (&config->path_new_name,       g_free);
-  g_clear_pointer (&config->path_export_path,    g_free);
-  g_clear_pointer (&config->path_import_path,    g_free);
+  g_clear_pointer (&config->vectors_new_name,    g_free);
+  g_clear_pointer (&config->vectors_export_path, g_free);
+  g_clear_pointer (&config->vectors_import_path, g_free);
 
   g_clear_object (&config->fill_options);
   g_clear_object (&config->stroke_options);
@@ -632,7 +626,9 @@ gimp_dialog_config_set_property (GObject      *object,
       break;
 
     case PROP_COLOR_PROFILE_PATH:
-      g_set_str (&config->color_profile_path, g_value_get_string (value));
+      if (config->color_profile_path)
+        g_free (config->color_profile_path);
+      config->color_profile_path = g_value_dup_string (value);
       break;
 
     case PROP_IMAGE_CONVERT_PROFILE_INTENT:
@@ -685,7 +681,9 @@ gimp_dialog_config_set_property (GObject      *object,
       break;
 
     case PROP_LAYER_NEW_NAME:
-      g_set_str (&config->layer_new_name, g_value_get_string (value));
+      if (config->layer_new_name)
+        g_free (config->layer_new_name);
+      config->layer_new_name = g_value_dup_string (value);
       break;
     case PROP_LAYER_NEW_MODE:
       config->layer_new_mode = g_value_get_enum (value);
@@ -728,32 +726,40 @@ gimp_dialog_config_set_property (GObject      *object,
       break;
 
     case PROP_CHANNEL_NEW_NAME:
-      g_set_str (&config->channel_new_name, g_value_get_string (value));
+      if (config->channel_new_name)
+        g_free (config->channel_new_name);
+      config->channel_new_name = g_value_dup_string (value);
       break;
     case PROP_CHANNEL_NEW_COLOR:
       g_clear_object (&config->channel_new_color);
       config->channel_new_color = gegl_color_duplicate (g_value_get_object (value));
       break;
 
-    case PROP_PATH_NEW_NAME:
-      g_set_str (&config->path_new_name, g_value_get_string (value));
+    case PROP_VECTORS_NEW_NAME:
+      if (config->vectors_new_name)
+        g_free (config->vectors_new_name);
+      config->vectors_new_name = g_value_dup_string (value);
       break;
 
-    case PROP_PATH_EXPORT_PATH:
-      g_set_str (&config->path_export_path, g_value_get_string (value));
+    case PROP_VECTORS_EXPORT_PATH:
+      if (config->vectors_export_path)
+        g_free (config->vectors_export_path);
+      config->vectors_export_path = g_value_dup_string (value);
       break;
-    case PROP_PATH_EXPORT_ACTIVE_ONLY:
-      config->path_export_active_only = g_value_get_boolean (value);
+    case PROP_VECTORS_EXPORT_ACTIVE_ONLY:
+      config->vectors_export_active_only = g_value_get_boolean (value);
       break;
 
-    case PROP_PATH_IMPORT_PATH:
-      g_set_str (&config->path_import_path, g_value_get_string (value));
+    case PROP_VECTORS_IMPORT_PATH:
+      if (config->vectors_import_path)
+        g_free (config->vectors_import_path);
+      config->vectors_import_path = g_value_dup_string (value);
       break;
-    case PROP_PATH_IMPORT_MERGE:
-      config->path_import_merge = g_value_get_boolean (value);
+    case PROP_VECTORS_IMPORT_MERGE:
+      config->vectors_import_merge = g_value_get_boolean (value);
       break;
-    case PROP_PATH_IMPORT_SCALE:
-      config->path_import_scale = g_value_get_boolean (value);
+    case PROP_VECTORS_IMPORT_SCALE:
+      config->vectors_import_scale = g_value_get_boolean (value);
       break;
 
     case PROP_SELECTION_FEATHER_RADIUS:
@@ -926,25 +932,25 @@ gimp_dialog_config_get_property (GObject    *object,
       g_value_set_object (value, config->channel_new_color);
       break;
 
-    case PROP_PATH_NEW_NAME:
-      g_value_set_string (value, config->path_new_name);
+    case PROP_VECTORS_NEW_NAME:
+      g_value_set_string (value, config->vectors_new_name);
       break;
 
-    case PROP_PATH_EXPORT_PATH:
-      g_value_set_string (value, config->path_export_path);
+    case PROP_VECTORS_EXPORT_PATH:
+      g_value_set_string (value, config->vectors_export_path);
       break;
-    case PROP_PATH_EXPORT_ACTIVE_ONLY:
-      g_value_set_boolean (value, config->path_export_active_only);
+    case PROP_VECTORS_EXPORT_ACTIVE_ONLY:
+      g_value_set_boolean (value, config->vectors_export_active_only);
       break;
 
-    case PROP_PATH_IMPORT_PATH:
-      g_value_set_string (value, config->path_import_path);
+    case PROP_VECTORS_IMPORT_PATH:
+      g_value_set_string (value, config->vectors_import_path);
       break;
-    case PROP_PATH_IMPORT_MERGE:
-      g_value_set_boolean (value, config->path_import_merge);
+    case PROP_VECTORS_IMPORT_MERGE:
+      g_value_set_boolean (value, config->vectors_import_merge);
       break;
-    case PROP_PATH_IMPORT_SCALE:
-      g_value_set_boolean (value, config->path_import_scale);
+    case PROP_VECTORS_IMPORT_SCALE:
+      g_value_set_boolean (value, config->vectors_import_scale);
       break;
 
     case PROP_SELECTION_FEATHER_RADIUS:
