@@ -23,9 +23,9 @@
 #include <gtk/gtk.h>
 
 #ifdef G_OS_WIN32
-#include <windows.h>
 #include <windef.h>
 #include <winbase.h>
+#include <windows.h>
 #endif
 
 #include "libgimpbase/gimpbase.h"
@@ -569,9 +569,6 @@ gimp_image_window_constructed (GObject *object)
   gimp_container_foreach (GIMP_CONTAINER (device_manager),
                           (GFunc) gimp_image_window_init_pad_foreach,
                           window);
-
-  /* This was only useful during construction and should not be further used. */
-  private->initial_monitor = NULL;
 }
 
 static void
@@ -1989,29 +1986,10 @@ gimp_image_window_hide_tooltip (GimpUIManager   *manager,
 static gboolean
 gimp_image_window_update_ui_manager_idle (GimpImageWindow *window)
 {
-  GimpImageWindowPrivate *private      = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
-  GimpImage              *active_image = NULL;
-  GimpContext            *context;
+  GimpImageWindowPrivate *private = GIMP_IMAGE_WINDOW_GET_PRIVATE (window);
 
   gimp_assert (private->active_shell != NULL);
 
-  context = gimp_get_user_context (private->active_shell->display->gimp);
-
-  if (context)
-    active_image = gimp_context_get_image (context);
-
-  /* Since we are running idle, it is possible this runs after the
-   * active display switched, and therefore we may call the wrong
-   * actions for an image. See #10441.
-   */
-  if ((active_image &&
-       (! private->active_shell->display || ! gimp_display_get_image (private->active_shell->display))) ||
-      active_image != gimp_display_get_image (private->active_shell->display))
-    {
-      private->update_ui_manager_idle_id = 0;
-      return G_SOURCE_REMOVE;
-    }
-
   gimp_ui_manager_update (menus_get_image_manager_singleton (private->active_shell->display->gimp),
                           private->active_shell->display);
 
@@ -2116,6 +2094,8 @@ gimp_image_window_switch_page (GtkNotebook     *notebook,
 
   gtk_window_set_title (GTK_WINDOW (window), shell->title);
 
+  gimp_display_shell_appearance_update (private->active_shell);
+
   if (gtk_widget_get_window (GTK_WIDGET (window)))
     {
       /*  we are fully initialized, use the window's current monitor
@@ -2125,7 +2105,7 @@ gimp_image_window_switch_page (GtkNotebook     *notebook,
                                         NULL /*new_entry_id*/,
                                         gimp_widget_get_monitor (GTK_WIDGET (window)));
     }
-  else if (private->initial_monitor != NULL)
+  else
     {
       /*  we are in construction, use the initial monitor; calling
        *  gimp_widget_get_monitor() would get us the monitor where the
@@ -2140,8 +2120,6 @@ gimp_image_window_switch_page (GtkNotebook     *notebook,
   gimp_context_set_display (gimp_get_user_context (private->gimp),
                             active_display);
 
-  gimp_display_shell_appearance_update (private->active_shell);
-
   gimp_image_window_update_ui_manager (window);
 
   gimp_image_window_update_tab_labels (window);
